#include <EEPROM.h>
#include <LiquidCrystal.h>
/* PINs:
 * 0 - COM RX 
 * 1 - COM TX
 * 2 - RX from GPS
 * 3 - PTT out
 * 4,5,6,7,8,9, A0 - used by LCD Keypad Shield (10 - PWM brightnes control - not used) 
 * 11, 10, 12 - (SDATA, SCLK, FSYNC)to AD98xx sintez
 * 13 - TONE on/off out
 * A1 - TONE on/off in manual CW mode (analog 0 is active - connecting to GND enables TONE)
 * A2 - AD98xx flag (if this input is connected to the GND then AD98xx sintez is in use)
 * A3 - free now ///zero is enable freq doubling (LF2: Fout = F x 2)
 * A4, A5 - free (may be for I2C)
 */
bool setF(unsigned long code);
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
#define BTN_UP   1
#define BTN_DOWN 2
#define BTN_LEFT 3
#define BTN_RIGHT 4
#define BTN_SELECT 5
#define BTN_NONE 10

#define MODE_CTRL 1
#define MODE_OP32 2
#define MODE_WSPR 3
#define MODE_WSPR15 4
#define MODE_QRSS 5
#define MODE_DFCW 6
#define OPERATION_MODES 6
#define MODE_MSG_QRSS 7
#define MODE_DOT_QRSS 8
#define MODE_MSG_DFCW 9
#define MODE_DOT_DFCW 10
#define MODE_TIME 11
#define MODE_BEACON 12
#define MODE_SUBMENU 13
#define MODE_CALL 14
#define MODE_QTH 15
#define MODE_PWR 16
#define MODE_GEN 17
#define MODE_LF2 18
#define MODE_CLR_EEPROM 19
#define MODE_MAX 20
#define MODE_TX MODE_MAX+1
#define MODE_WAIT MODE_MAX+2
bool submenu;

#define MSG_LEN_MAX 16
#define M_LEN_MAX 5
#define CALL_LEN_MAX 6
#define CC_LEN_MAX 5
#define QTH_LEN_MAX 4
#define QQ_LEN_MAX 5

char mode=MODE_CTRL, mode_old=0, mode_next=0;
char mode_start;
char msgq[MSG_LEN_MAX+2]; 
char msgd[MSG_LEN_MAX+2];
char msgq_pos=0, msgd_pos=0;
char m[M_LEN_MAX+2];
char m_pos=0;

char call[CALL_LEN_MAX+2];
char call_pos=0;
char cc[CC_LEN_MAX+2];
char cc_pos=0;

char qth[QTH_LEN_MAX+2];
volatile char qth_pos=0;
char qq[QQ_LEN_MAX+2];
char qq_pos=0;
byte clr_eeprom=0;

byte list_n=0;
int pwr;
const byte pwr_list[19]={0,3,7,10,13,17,20,23,27,30,33,37,40,43,47,50,53,57,60};
char n_pwr;

unsigned int updwcnt;
double freq;
byte btn_old=0, last_btn=0;
unsigned long Code, Code_old;
unsigned long CodeQRSS, CodeDFCW, CodeWSPR, CodeWSPR15,CodeOP32;
double freqQRSS, freqDFCW, freqWSPR, freqWSPR15, freqOP32;
unsigned long dc=1UL;
unsigned long Crystal;
byte Cycles;
double Step;
byte LF2;

#define CRYSTAL_LIST_N 13
const unsigned long Crystal_list[CRYSTAL_LIST_N]={5000000UL,10000000UL,12000000UL,12800000UL,12800175UL,13000000UL,13500000UL,15000000UL,15600000UL,16000000UL,16384000UL,20000000UL,25000000UL};

byte DDS_VER;
bool DDSAD;
bool BIT32;
bool GMDDS;
byte GMTONE, GMPTT, GMPH;
byte b0_32; 

volatile byte IsToneOn; 
volatile byte IsPttOn;

int dot_qrss;
int dot_dfcw;

#define TX_LEN_MAX 255
char tx[TX_LEN_MAX+2];
volatile int tx_pos;
volatile int tx_len;
volatile unsigned long timer_interval;
unsigned int tx_dc=1UL;
double tx_df;

char t_min, t_sec;
char t_set_min, t_set_sec;
unsigned long t_dif;
bool timeIsSet=false;

#define BCN_LOAD 0
#define BCN_RUN  1
#define BCN_QRSS 2
#define BCN_DFCW 3
#define BCN_OP32 4
#define BCN_WSPR 5
#define BCN_WSPR15 6
#define BCN_PAUSE_1 7
#define BCN_PAUSE_5 8
#define BCN_RPT 7
#define BCN_SAVE 8
#define BCN_CLR 9
#define BCN_MAX 10

bool BEACON=false;
#define BCN_LIST_LEN 15
char bcn_list[BCN_LIST_LEN+2];
byte bcn_pos;
byte bcn=1;

#define EEPROM_CALL_ADR 0
#define EEPROM_QTH_ADR      (EEPROM_CALL_ADR+sizeof(call))
#define EEPROM_PWR_ADR      (EEPROM_QTH_ADR+sizeof(qth))
#define EEPROM_CODE_ADR     (EEPROM_PWR_ADR+sizeof(pwr))
#define EEPROM_MSG_DFCW_ADR (EEPROM_CODE_ADR+sizeof(freq))
#define EEPROM_MODE_ADR     (EEPROM_MSG_DFCW_ADR+sizeof(msgd))
#define EEPROM_CRYSTAL_ADR  (EEPROM_MODE_ADR+sizeof(mode))
#define EEPROM_CODEQRSS_ADR (EEPROM_CRYSTAL_ADR+sizeof(Crystal))
#define EEPROM_CODEDFCW_ADR (EEPROM_CODEQRSS_ADR+sizeof(freqQRSS))
#define EEPROM_CODEWSPR_ADR (EEPROM_CODEDFCW_ADR+sizeof(freqDFCW))
#define EEPROM_CODEOP32_ADR (EEPROM_CODEWSPR_ADR+sizeof(freqWSPR))
#define EEPROM_CODEWSPR15_ADR (EEPROM_CODEOP32_ADR+sizeof(freqOP32))
#define EEPROM_DOTQRSS_ADR  (EEPROM_CODEWSPR15_ADR+sizeof(freqWSPR15))
#define EEPROM_DOTDFCW_ADR  (EEPROM_DOTQRSS_ADR+sizeof(dot_qrss))
#define EEPROM_BCNLIST_ADR  (EEPROM_DOTDFCW_ADR+sizeof(dot_dfcw))
#define EEPROM_MSG_QRSS_ADR (EEPROM_BCNLIST_ADR+sizeof(bcn_list))
#define EEPROM_LF2_ADR      (EEPROM_MSG_QRSS_ADR+sizeof(msgq))
/*
extern int __bss_end;
extern void *__brkval;
 
// Функция, возвращающая количество свободного ОЗУ (RAM)
int memoryFree()
{
   int freeValue;
   if((int)__brkval == 0)
      freeValue = ((int)&freeValue) - ((int)&__bss_end);
   else
      freeValue = ((int)&freeValue) - ((int)__brkval);
   return freeValue;
}
*/
//// ф-ции для AD9834
#define hibyte(x) (unsigned int)(x >> 14)    // макрос определения старшего разряда
#define lobyte(x) (unsigned int)(x & 0x3FFF) // макрос определения младшего разряда
#define SDATA  11   // данные DDS
#define SCLK   10     // строб DDS
#define FSYNC  12    // обновление данных в DDS
#define AD98XX_FLAG_PIN A2 // аналоговый вход 1 - если он равен 0 (заземлен) - это означает подключение AD9833
#define AD98XX_PTT  3     // управление PTT
#define AD98XX_TONE 13    // управление тоном
//Инициализация пинов
void setup_AD98xx_pins(void)
{
  pinMode(SCLK, OUTPUT);
  pinMode(FSYNC, OUTPUT);
  pinMode(SDATA, OUTPUT);
  pinMode(AD98XX_PTT, OUTPUT);
  pinMode(AD98XX_TONE, OUTPUT);
  digitalWrite(FSYNC,HIGH);
  digitalWrite(SCLK,HIGH); 
  digitalWrite(SDATA,LOW);
}
// Запись 16 бит в DDS. Первым записывается старший бит. 
void dds_write(unsigned int word)
{
   char i;    
   digitalWrite(FSYNC,LOW);  //FSYNC = 0;    
   for(i = 15; i >= 0; i--)     
   {         
     if ((word >> i) & 1)
       digitalWrite(SDATA,HIGH); //SDATA = 1;
     else
       digitalWrite(SDATA,LOW); //SDATA = 0; 
     digitalWrite(SCLK,LOW);    //SCLK = 0;
     delayMicroseconds(10);     
     digitalWrite(SCLK,HIGH);   //SCLK = 1;
   }
  digitalWrite(FSYNC,HIGH);     //FSYNC = 1;
}

//  Запись частоты в DDS.
void dds_set(unsigned long f)
{ 
   dds_write((lobyte(f) | 0x4000));  // Записываем младшие 14 бит добавляя в начало адрес регистра 01 bin
   dds_write((hibyte(f) | 0x4000));  // Записываем старшие 14 бит добавляя в начало адрес регистра 01 bin
   dds_write(0xC000); // информация о фазе
   dds_write(0x2000); // выбор первого регистра частоты, обнуление бита RESET
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//For GPS RX
#define GPS_RX_PIN 2 // пин для приема NMEA from GPPS, на нем есть прерывание
#define GPS_1B_DUR 104  // =1000000 / GPS_RX_SPEED - длительность одной посылки в мкс (104 для 9600, 208 для 4800)
#define GPS_LEN 50
bool gps_on=false; 
volatile bool gps_ready=false;
volatile bool gps_detected=false;
char gps_s[GPS_LEN+2];
byte gps_cnt;
double lat,lon;

void latlon_to_loc(void)
{
 qth[0]=65 + lon/20;
 qth[1]=65 + lat/10;
 qth[2]=48 + (lon-(qth[0]-65)*20)/2;
 qth[3]=48 + (lat-(qth[1]-65)*10); 
 qth_pos=3;
}

void gps_read()
{ char c=0;
  static char comma;
  static byte comma_pos;
  static char s_lat[6], s_lon[7];
  delayMicroseconds(3*GPS_1B_DUR/2);// ждем первый бит
  for(byte i=0; i<8; i++)
  {
    bitWrite(c,i,digitalRead(GPS_RX_PIN));
    delayMicroseconds(GPS_1B_DUR);
  }
  if(digitalRead(GPS_RX_PIN)==LOW){ gps_detected=false; gps_ready=false; return;} //нет стопового бита - ошибка
  if(c<32||c>126) return;
  if(c=='$') {
    memset(gps_s,0,sizeof(gps_s)); 
    gps_cnt=0; gps_detected=true; comma=0; comma_pos=0;
  }
  else{ 
   gps_s[gps_cnt]=c;gps_cnt++;if(gps_cnt>=GPS_LEN)gps_cnt=GPS_LEN-1;
  }
  // найти нужный заголовок и взять время
  if(gps_cnt==12)
  {   if(strncmp(gps_s,"GPRMC,",6)==0 || strncmp(gps_s,"GNRMC,",6)==0){
        t_set_min=10*(gps_s[8]-48) + (gps_s[9]-48);
        t_set_sec=10*(gps_s[10]-48) + (gps_s[11]-48);
        comma=0; comma_pos=0;
      }
  }
   //теперь определить QTH-loc
   /// $GPRMC,144326.00,A,5107.0017737,N,11402.3291611,W,0.080,323.3,210307,0.0,E,A*20
   /// $GPRMC,123519,A,3840.038,N,00919.000,W,022.4,084.4,230394,003.1,W*6A
  if(gps_cnt>12)
  {  if(c==',') {comma++; comma_pos=gps_cnt;}
     switch(comma)
      { case 1: //  поле - признак A/V
              if(gps_cnt-comma_pos==1)
              { if(gps_s[comma_pos]=='A' || gps_s[comma_pos]=='a') gps_ready=true;
                else {gps_ready=false;comma=6;}
              }
              break;
        case 2: //  поле - широта
              if(gps_cnt-comma_pos==4){s_lat[0]=gps_s[comma_pos];s_lat[1]=gps_s[comma_pos+1];s_lat[2]=gps_s[comma_pos+2];s_lat[3]=gps_s[comma_pos+3];}
              break;
        case 3: //  поле - знак широты
              if(gps_cnt-comma_pos==1)
              {switch(gps_s[comma_pos])
               { case 'N': case 'n': s_lat[4]='N'; break;
                 case 'S': case 's': s_lat[4]='S'; break;
                 default: {gps_ready=false;comma=6;}
               }
              }
              break;
        case 4: //  поле - долгота
              if(gps_cnt-comma_pos==5){s_lon[0]=gps_s[comma_pos];s_lon[1]=gps_s[comma_pos+1];s_lon[2]=gps_s[comma_pos+2];s_lon[3]=gps_s[comma_pos+3];s_lon[4]=gps_s[comma_pos+4];}
              break;
        case 5: //  поле - знак долготы
              if(gps_cnt-comma_pos==1)
              { switch(gps_s[comma_pos])
                { case 'W': case 'w': s_lon[5]='W'; break;
                  case 'E': case 'e': s_lon[5]='E'; break;
                  default: gps_ready=false;
                }
                if(gps_ready){
                  setmytime(); //устанавливаем, если время правильное
                  lat=(s_lat[0]-48)*10 + (s_lat[1]-48) + ((s_lat[2]-48)*10 + (s_lat[3]-48))/60.0;
                  if(s_lat[4]=='S')lat=90-lat; else lat=90+lat;
                  lon=(s_lon[0]-48)*100 + (s_lon[1]-48)*10 + (s_lon[2]-48) + ((s_lon[3]-48)*10 + (s_lon[4]-48))/60.0;
                  if(s_lon[5]=='W')lon=180-lon; else lon=180+lon;;
                  latlon_to_loc();
                }
                comma++;
              }
              break;
      }
   }
}

void enable_gps(void)
{ if(gps_on) return;
  gps_on=true; gps_ready=false; gps_detected=false;
  pinMode(GPS_RX_PIN,INPUT);
  attachInterrupt(digitalPinToInterrupt(GPS_RX_PIN), gps_read, FALLING);
}
void disable_gps(void)
{ if(!gps_on) return;
  detachInterrupt(digitalPinToInterrupt(GPS_RX_PIN));
  gps_on=false; gps_ready=false; gps_detected=false;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define LEN_ABC 37
char abc[LEN_ABC+1]={"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ "};
char * dfcw[]={"-----","_----","__---","___--","____-","_____","-____","--___","---__","----_",
               "_-","-___","-_-_","-__","_","__-_","--_","____","__","_---","-_-","_-__","--","-_","---","_--_",
               "--_-","_-_","___","-","__-","___-","_--","-__-","-_--","--__",
               "-___-"};
//----------------------------------------------------------------
char dfcw_to_char(char mm[])
{
 for(int i=0; i<LEN_ABC; i++)
 {
  if(strcmp(mm,dfcw[i])==0) return abc[i]; 
 }
 return 0;
}
//------------------------------------------------------------------
char * char_to_dfcw(char c)
{ 
  for(int i=0; i<LEN_ABC;i++)
  {
    if(c==abc[i]) return dfcw[i];
  }
  return " ";
}
void recomputeCode(void)
{
  Code=(unsigned long)freq/Step; if(freq-Code*Step>=0.5)Code++;
  CodeQRSS=(unsigned long)freqQRSS/Step; if(freqQRSS-CodeQRSS*Step>=0.5)CodeQRSS++;
  CodeDFCW=(unsigned long)freqDFCW/Step; if(freqDFCW-CodeDFCW*Step>=0.5)CodeDFCW++;
  CodeOP32=(unsigned long)freqOP32/Step; if(freqOP32-CodeOP32*Step>=0.5)CodeOP32++;
  CodeWSPR=(unsigned long)freqWSPR/Step; if(freqWSPR-CodeWSPR*Step>=0.5)CodeWSPR++;
  CodeWSPR15=(unsigned long)freqWSPR15/Step; if(freqWSPR15-CodeWSPR15*Step>=0.5)CodeWSPR15++;
}
//-------------------------------------------------------------------
void set28bit(void)
{ 
  Cycles=1;
  Step=Crystal;//Cycles;
  Step=Step/16384UL;
  Step=Step/16384UL;
  recomputeCode();
  BIT32=false;
  GMDDS=false;
  DDS_VER=0;
  DDSAD=true;
}
//---------------------------------------------------------------------------

//-------------------------------------------------------------------
void set32bit(void)
{ 
  //if(Cycles==12) return;
  Cycles=12;
  Step=Crystal/Cycles;
  Step=Step/65536UL;
  Step=Step/65536UL;
  recomputeCode();
  BIT32=true;
  if(GMDDS){Serial.begin(9600);Serial.setTimeout(100);}
  GMDDS=false;
  DDSAD=false;
}
//---------------------------------------------------------------------------
void set24bit(void)
{ 
  //if(Cycles==11) return;
  Cycles=11;
  Step=Crystal/Cycles;
  Step=Step/4096;
  Step=Step/4096;
  recomputeCode();
  BIT32=false;
  if(GMDDS){Serial.begin(9600);Serial.setTimeout(100);}
  GMDDS=false;
  DDSAD=false;
}
//---------------------------------------------------------------------------
void setgmdds(void)
{ //if(Cycles==10) return;
  Cycles=10; BIT32=true;
  Step=Crystal/Cycles;
  Step=Step/65536UL;
  Step=Step/65536UL;
  DDS_VER=0;
  recomputeCode();
  if(!GMDDS){Serial.begin(115200);Serial.setTimeout(100);}
  GMDDS=true;
  DDSAD=false;
}
//---------------------------------------------------------------------------
bool toneON(void)
{ unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
  digitalWrite(AD98XX_TONE,HIGH);
  IsToneOn=true;
  if(DDSAD){ dds_write(Code); return true;}
  if(GMDDS)
  { GMTONE=0;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    bool result=false;
    Serial.write(c,4);
    result=readF();
    return result;
  }
  c[0]='1';
  unsigned long N=1;
  Serial.write(c,1);
  return true;
}
//---------------------------------------------------------------------------
bool toneOFF(void)
{ unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
  digitalWrite(AD98XX_TONE,LOW);
  IsToneOn=false;
  if(DDSAD) {dds_write(0);return true;}
  if(GMDDS)
  { GMTONE=1;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    bool result=false;
    Serial.write(c,4);
    result=readF();
    return result;
  }
  c[0]='0';
  Serial.write(c,1);
  return true;
}
//---------------------------------------------------------------------------
bool pttON(void)
{ bool result=false;
  unsigned char c[5];
  c[1]=c[2]=c[3]=c[4]=0;
  digitalWrite(AD98XX_PTT,HIGH);
  IsPttOn=true;

  if(DDSAD)return true;
  
  if(GMDDS)
  { 
    GMPTT=1;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    Serial.write(c,4);
    result=readF();
  }
  else{
   c[0]='T';
   Serial.write(c,1);
   result=readF();
     }
  return result;
}

bool pttOFF(void)
{ bool result=false;
 unsigned char c[5];
 c[1]=c[2]=c[3]=c[4]=0;
 digitalWrite(AD98XX_PTT,LOW);
 IsPttOn=false;

 if(DDSAD) return true;
 
  if(GMDDS)
  { GMPTT=0;
    c[0]=0x80 | (GMPTT<<4) | (GMTONE<<5);
    Serial.write(c,4);
    result=readF();
  }
   else{
    c[0]='R';
    Serial.write(c,1);
    result=readF();
       }
  return result;
}
//-----------------------------------------------------------------------
bool setF4(unsigned long code[])
{
 char c[14];
 if(LF2) for(int i=0; i<3; i++) code[i]=code[i]<<1;
   c[0]='L';
   c[1]=0xff & (code[0]>>16);
   c[2]=0xff & (code[0]>>8);
   c[3]=0xff & (code[0]>>0);
   c[4]=0xff & (code[1]>>16);
   c[5]=0xff & (code[1]>>8);
   c[6]=0xff & (code[1]>>0);
   c[7]=0xff & (code[2]>>16);
   c[8]=0xff & (code[2]>>8);
   c[9]=0xff & (code[2]>>0);
   c[10]=0xff & (code[3]>>16);
   c[11]=0xff & (code[3]>>8);
   c[12]=0xff & (code[3]>>0);
   c[13]=0;
   bool result=false;
   for(int i=0; i<13; i++)
   {
     Serial.write(&c[i],1);
     delayMicroseconds(10000);
   }
   result=readF();
   return result;
}

//-----------------------------------------------------------------------
bool setFastF(byte f)
{
 if(GMDDS) return false;
  unsigned char c=0;
  switch(f)
  { case 0: c=0xf0; break;
    case 1: c=0xf1; break;
    case 2: c=0xf2; break;
    case 3: c=0xf3; break;
    case 4: c=0xf3; break;
  }
  Serial.write(&c,1);
  return true;
}
//-----------------------------------------------------------------------

int detectButton() {
  static unsigned long tb;
  int keyAnalog =  analogRead(A0);
  if (keyAnalog < 100) {
    // Значение меньше 100 – нажата кнопка right
    return BTN_RIGHT;
  } else if (keyAnalog < 200) {
    // Значение больше 100 (иначе мы бы вошли в предыдущий блок результата сравнения, но меньше 200 – нажата кнопка UP
    return BTN_UP;
  } else if (keyAnalog < 400) {
    // Значение больше 200, но меньше 400 – нажата кнопка DOWN
    return BTN_DOWN;
  } else if (keyAnalog < 600) {
    // Значение больше 400, но меньше 600 – нажата кнопка LEFT
    return BTN_LEFT;
  } else if (keyAnalog < 800) {
    // Значение больше 600, но меньше 800 – нажата кнопка SELECT
    return BTN_SELECT;
  } else {
    // Все остальные значения (до 1023) будут означать, что нажатий не было
    return BTN_NONE;
  }
}
void clearLine(int line){
  lcd.setCursor(0, line);
  lcd.print(F("                "));
  lcd.setCursor(0, line);
}

void printError(char err[])
{
 clearLine(1); lcd.print(F("ERR:"));lcd.print(err); /*delay(5000);*/
}

void printFreq(unsigned long CC){
  double F=CC*Step;
  if(mode>OPERATION_MODES && mode!=MODE_TX)return;
  lcd.setCursor(0,0);lcd.print(F("            "));
  lcd.setCursor(0,0);lcd.print(F);
  //lcd.print(F("  "));
}
void printTonePtt(void)
{
  clearLine(1);
  if(IsToneOn==1) lcd.print(F("TONE=ON "));else lcd.print(F("TONE=OFF"));
  lcd.setCursor(9,1); 
  if(IsPttOn==1) lcd.print(F("PTT=ON "));else lcd.print(F("PTT=OFF"));
}

void paintTX(void)
{ 
  unsigned int percent=99*tx_pos/tx_len;
  unsigned int time_to_end=(tx_len-tx_pos)*timer_interval/1000;
  printFreq(Code);
  clearLine(1);
  if(IsToneOn)lcd.print(F("TX "));else lcd.print(F("tx ")); 
  if(percent<10)lcd.setCursor(4,1);
  else lcd.setCursor(3,1); 
  lcd.print(percent);lcd.print(F("% "));
  lcd.print(time_to_end);
  lcd.print(F("s"));
  lcd.setCursor(13,1);
  if(BEACON) lcd.print(F("BCN"));
}

void printMode(int mod){
  lcd.setCursor(12,0);
  //if(mode<=OPERATION_MODES) EEPROM.put(EEPROM_MODE_ADR,mode);;
  if(mod==MODE_WAIT || mod==MODE_TIME || mod==MODE_QTH) enable_gps(); else disable_gps();
  switch(mod){
    case MODE_QRSS: printFreq(CodeQRSS); setF(CodeQRSS); lcd.setCursor(12,0);lcd.print(F("QRSS")); 
                    clearLine(1); lcd.print(msgq);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_qrss);lcd.print('s');
                    break;
    case MODE_DFCW: printFreq(CodeDFCW); setF(CodeDFCW);lcd.setCursor(12,0);lcd.print(F("DFCW")); 
                    clearLine(1); lcd.print(msgd);lcd.setCursor(12,1);lcd.print(F(" "));lcd.print(dot_dfcw);lcd.print('s');
                    break;
    case MODE_WSPR: printFreq(CodeWSPR); setF(CodeWSPR);lcd.setCursor(12,0);lcd.print(F("WSPR")); 
                    clearLine(1); lcd.print(call);
                                  lcd.setCursor(7,1); lcd.print(qth);
                                  lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
                    break;
    case MODE_WSPR15: printFreq(CodeWSPR15); setF(CodeWSPR15); lcd.setCursor(12,0);lcd.print(F("WR15")); 
                    clearLine(1); lcd.print(call);
                                  lcd.setCursor(7,1); lcd.print(qth);
                                  lcd.setCursor(12,1); lcd.print(pwr); lcd.print(F("dB"));
                    break;
    case MODE_OP32: printFreq(CodeOP32); setF(CodeOP32);lcd.setCursor(12,0); lcd.print(F("OP32")); 
                    clearLine(1); lcd.print(call);
                    break;
    case MODE_CTRL:  printFreq(Code); lcd.setCursor(12,0); lcd.print(F("  CW")); 
                    printTonePtt();
                    break;
    case MODE_MSG_QRSS:  clearLine(0); lcd.print(F("MSG QRSS:")); 
                    lcd.print(m);
                    clearLine(1); lcd.print(msgq);
                    break;
    case MODE_MSG_DFCW:  clearLine(0); lcd.print(F("MSG DFCW:")); 
                    lcd.print(m);
                    clearLine(1); lcd.print(msgd);
                    break;
    case MODE_CALL: clearLine(0); lcd.print(F("CALL:")); 
                    lcd.print(cc);
                    clearLine(1); lcd.print(call);
                    break;
    case MODE_QTH:  clearLine(0); clearLine(1); lcd.setCursor(7,0); lcd.print(F("LOC:")); 
                    lcd.print(qq); 
                    lcd.setCursor(7,1); lcd.print(qth);
                    break;
    case MODE_PWR:  clearLine(0); clearLine(1); lcd.setCursor(12,0);lcd.print(F("PWR:")); 
                    lcd.setCursor(12,1); lcd.print(pwr); lcd.print(F("dB"));
                    break;
    case MODE_DOT_QRSS:  clearLine(0); clearLine(1); lcd.setCursor(7,0);lcd.print(F("QRSS DOT:")); 
                    lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
                    break;
    case MODE_DOT_DFCW:  clearLine(0); clearLine(1); lcd.setCursor(7,0);lcd.print(F("DFCW DOT:")); 
                    lcd.setCursor(12,1); lcd.print(dot_dfcw); lcd.print(F("s"));
                    break;
    case MODE_GEN:  clearLine(0);lcd.print(F("CRYSTAL ="));
                    clearLine(1);lcd.print(Crystal); lcd.print(F(" Hz"));
                    break;
    case MODE_CLR_EEPROM:
                    clearLine(0); lcd.print(F("ERASE SETTINGS ?"));
                    clearLine(1); lcd.setCursor(7,1); lcd.print(F("NO"));
                    break;
    case MODE_TX:   clearLine(0);clearLine(1);
                    paintTX();
                    break;
    case MODE_TIME:  clearLine(0); lcd.print(F("SET TIME [mm:ss]"));
                    if(timeIsSet ){ t_set_min=t_min; t_set_sec=t_sec;}
                    clearLine(1); printTime(t_set_min,t_set_sec);
                    break;
    case MODE_WAIT:  clearLine(0); lcd.print(F("WAITING WSPR..."));
                    clearLine(1); printTime(t_min, t_sec);
                    break;
    case MODE_BEACON:  clearLine(0); lcd.print(F("BEACON:"));
                    printBcnMenu(bcn);
                    clearLine(1); printBeaconList(); 
                    break;
    case MODE_SUBMENU: clearLine(0); if(submenu) lcd.print(F("Exit SETUP...")); else lcd.print(F("SETUP..."));
                    clearLine(1);
                    break;
    case MODE_LF2:  lf2updw();
                    clearLine(1);
                    break;
    
    default:   ;//lcd.print(F("????"));
  } 
}
/*
void printCode(unsigned long cd){
  lcd.setCursor(0,0);
  lcd.print(cd,HEX);
}
*/
void printBeaconList(void)
{
 clearLine(1);
 lcd.print(bcn_list); 
}

void printTime(byte mm, byte ss)
{
 byte m1,m2,s1,s2;
 m1=mm%10; m2=(mm-m1)/10;
 s1=ss%10; s2=(ss-s1)/10;
 lcd.setCursor(0,1);
 if(gps_ready) lcd.print(F("GPS = "));
 else {if(gps_detected) lcd.print(F("(GPS) "));
       else  lcd.print(F("      ")); 
 }
 lcd.setCursor(6,1); 
 lcd.print(m2,DEC);
 lcd.print(m1,DEC);
 lcd.print(':');
 lcd.print(s2,DEC);
 lcd.print(s1,DEC);
}

bool readF(void)
{
   unsigned char s[8];
   memset(s,0,sizeof(s));
   unsigned long N=1 ;
   if(GMDDS)
   {N=Serial.readBytes(s,4);
    if(N==0) return false;
    if(N!=4) return false;
             
     unsigned long x1,x2,x3,x4;
     unsigned long y=0;
     x1=x2=x3=x4=0;
     y=s[0]; x1=(y<<24)&0xff000000; 
     y=s[1]; x2=(y<<16)&0x00ff0000; 
     y=s[2]; x3=(y<<8)&0x0000ff00; 
     x4=(s[3])&0x000000ff;
     if(s[0]&0x80) //это команда и фаза
     {
     }
     else
     {
      Code=x1 + x2  + x3 +  x4;
      if(LF2) Code=Code>>1;
     }
    return true;
   }

   N=Serial.readBytes(s,1);
   if(N==0) return false;
   if(s[0]=='F')
   { 
M1:    
     unsigned long x1,x2,x3; 
     N=Serial.readBytes(&s[1],5); //дочитать байты
     if(N!=5) return false;
     if(s[1]>0)
     { DDS_VER=(s[1]&0xe0)>>5;
       if(s[1]&0x01) IsPttOn=1;//PTT отобразить
       else IsPttOn=0;
       if(s[1]&0x10) set32bit();//32 bit mode
       else set24bit();
     }
     else
     {DDS_VER=0;
      set24bit();
     }
     unsigned long y=0;
     x1=x2=x3=0;
     if(BIT32){
       y=s[2]; x1=(y<<24)&0xff000000; 
       y=s[3]; x2=(y<<16)&0x00ff0000; 
       y=s[4]; x3=(y<<8)&0x0000ff00;
     }
     else {
      y=s[2]; x1=(y<<16)&0xff0000; 
      y=s[3]; x2=(y<<8)&0x00ff00; 
      x3=(s[4])&0x0000ff;
     }
     Code=x1 + x2  + x3 +  b0_32;
     if(LF2) Code=Code>>1;
   }
   else
   {
    if(N==0) return false;
    int rr=Serial.available();
    for(int i=0; i<rr; i++)
    {
      char cccc=0; Serial.readBytes(&cccc,1);
      if(cccc=='F') goto M1;
    }
    Serial.begin(9600);
    Serial.setTimeout(300);
    return false;
   }
   return true;
}
bool setF(unsigned long code/*, bool invPh*/)
{ Code=code;
  if(LF2) code=code<<1;
   unsigned long CC=code;
    unsigned char c[8];
  memset(c,0,sizeof(c));
  bool result=false;
  if(DDSAD)
  {
    dds_set(code); //Code=code;
    return true;
  }
  if(GMDDS)
  { /*if(invPh)
    {
     c[0]=0xc0 | (GMPTT<<4) | (GMTONE<<5);
     c[1]=(0xff00 & GMPH)>>8;
     c[2]=(0x00ff & GMPH);
    }
    else*/
    //{
     CC=code; c[0]=0x7f & (CC>>24);
     CC=code; c[1]=0xff & (CC>>16);
     CC=code; c[2]=0xff & (CC>>8);
     CC=code; c[3]=0xff & (CC);
    //}
     result=false;
     Serial.write(c,4);
     result=readF();
   return result;
  }
   c[0]='s';
   if(BIT32)
   { CC=code; c[1]=0xff & (CC);
     b0_32=c[1];
     CC=code; c[2]=0xff & (CC>>24);
     CC=code; c[3]=0xff & (CC>>16);
     CC=code; c[4]=0xff & (CC>>8);
   }
   else
   { c[1]=0;
     b0_32=c[1];
     CC=code; c[2]=0xff & (CC>>16);
     CC=code; c[3]=0xff & (CC>>8);
     CC=code; c[4]=0xff & (CC>>0);
   }
   c[5]=0;
   result=true;
   Serial.write(c,5);
   result=readF();
   if(!result) printError("NO DDS");
   return result;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const int order = 16;
const unsigned long polynom = 0x8005UL;
const unsigned long crcxor = 0x0UL;
//const int refin = 1;
//const int refout = 1;

unsigned long crcmask;
//unsigned long crcinit_direct;
unsigned long crchighbit;

unsigned long reflect (unsigned long crc, int bitnum) {
  // reflects the lower 'bitnum' bits of 'crc'
  unsigned long i, j=1, crcout=0;
  for (i=(unsigned long)1<<(bitnum-1); i; i>>=1) {
    if (crc & i) crcout|=j;
    j<<= 1;
  }
  return (crcout);
}

unsigned long crcbitbybitfast(char* p, unsigned long len) {

  // fast bit by bit algorithm without augmented zero bytes.
  // does not use lookup table, suited for polynom orders between 1...32.

  unsigned long i, j, c, b;
  unsigned long crc = 0UL;//crcinit_direct;

  for (i=0; i<len; i++) {
    c = (unsigned long)*p++;
    /*if (refin)*/ c = reflect(c, 8);

    for (j=0x80; j; j>>=1) {
      b = crc & crchighbit;
      crc<<= 1;
      if (c & j) b^= crchighbit;
      if (b) crc^= polynom;
    }
  }
  /*if (refout)*/ crc=reflect(crc, order);
  crc^= crcxor;
  crc&= crcmask;
  return(crc);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int opera_coding(void)
{memset(tx,0,sizeof(tx)); tx_pos=0;
 char c[7];
 memset(c,0,sizeof(c));
 strcpy(c,call);
  // подготовить позывной
 if(strlen(c)<6){
  if(!isdigit(c[2])) {memmove(&c[1],&c[0],5*sizeof(char));c[0]=' ';c[6]=0;}
 }
 byte len=strlen(c);
 switch(len)
 { case 3: c[3]=' ';c[4]=' ';c[5]=' ';c[6]=0;break;
   case 4: c[4]=' ';c[5]=' ';c[6]=0; break;
   case 5: c[5]=' ';c[6]=0; break;
 }
 if(!isdigit(c[2])){printError("BAD CALL"); return 0;}

 
 // encoding mapping
 // ( ( ( ( (1) * 36 + 0) * 10 + 1) * 27 + 1) * 27 + 1) * 27 + 0
 if(c[0]==' ') c[0]=0;
 if(c[0]>47 && c[0]<58) c[0]=c[0]-21; // 0...9
 if(c[0]>64 && c[0]<91) c[0]=c[0]-64; // A...Z

 if(c[1]>47 && c[1]<58) c[1]=c[1]-22; // 0...9
 if(c[1]>64 && c[1]<91) c[1]=c[1]-65; // A...Z

 if(c[2]>47 && c[2]<58) c[2]=c[2]-48; // 0...9

 if(c[3]==' ') c[3]=0;
 if(c[3]>64 && c[3]<91) c[3]=c[3]-64; // A...Z

 if(c[4]==' ') c[4]=0;
 if(c[4]>64 && c[4]<91) c[4]=c[4]-64; // A...Z

 if(c[5]==' ') c[5]=0;
 if(c[5]>64 && c[5]<91) c[5]=c[5]-64; // A...Z

 uint64_t map=0ULL;
 //map=(((((c[0])*36 + c[1]) * 10 + c[2]) * 27 + c[3]) * 27 + c[4]) * 27 + c[5];
 map=c[1];
 map=map + c[0]*36;
 map=map*10;
 map=map + c[2];
 map=map*27;
 map=map + c[3];
 map=map*27;
 map=map+c[4];
 map=map*27;
 map=map+c[5];
   
 unsigned long crc;
 char msg51[51];
 char arr[47];
 for(int i=0; i<28; i++) {if(1 & (map>>i)) arr[27-i]='1'; else arr[27-i]='0';}
 crcmask = ((((unsigned long)1<<(order-1))-1)<<1)|1;
 crchighbit = (unsigned long)1<<(order-1);
 crc=crcbitbybitfast(arr, 28);
 unsigned int crc_low,crc_high;
 crc_low=crc&0xff;
 crc_high=(crc>>8)&0xff;
 if(crc_low==0)crc_low=0x1b;
 if(crc_high==0)crc_high=0x2b;
 crc=(crc_low<<8)^crc_high;
 for(int i=0; i<16; i++) {if(1 & (crc>>i)) arr[27+16-i]='1'; else arr[27+16-i]='0';}
 crc=crcbitbybitfast(arr, 28+16);
 crc_low=crc&0xff;
 crc_high=(crc>>8)&0xff;
 if(crc_low==0)crc_low=0x1b;
 if(crc_high==0)crc_high=0x2b;
 crc=(crc_low<<8)^crc_high;
 crc=crc&0x07;
 for(int i=0; i<3; i++) {if(1 & (crc>>i)) arr[27+16+3-i]='1'; else arr[27+16+3-i]='0';}
 char nvect[]="111000010101011111100110110100000001100100010101011";
 for(int i=0; i<28+16+3; i++) msg51[i+4]=arr[i];
 for(int i=0; i<4; i++) msg51[i]='0';
 for(int i=0; i<51; i++)
 { if(msg51[i]==nvect[i]) msg51[i]='0';
   else msg51[i]='1';
 }

 char wh_coded[17][8];
 for(int i=0; i<51; i=i+3)
 {
  if(strncmp(&msg51[i],"000",3)==0)strncpy(&wh_coded[i/3][0],"0000000",7);
  if(strncmp(&msg51[i],"001",3)==0)strncpy(&wh_coded[i/3][0],"1010101",7);
  if(strncmp(&msg51[i],"010",3)==0)strncpy(&wh_coded[i/3][0],"0110011",7);
  if(strncmp(&msg51[i],"011",3)==0)strncpy(&wh_coded[i/3][0],"1100110",7);
  if(strncmp(&msg51[i],"100",3)==0)strncpy(&wh_coded[i/3][0],"0001111",7);
  if(strncmp(&msg51[i],"101",3)==0)strncpy(&wh_coded[i/3][0],"1011010",7);
  if(strncmp(&msg51[i],"110",3)==0)strncpy(&wh_coded[i/3][0],"0111100",7);
  if(strncmp(&msg51[i],"111",3)==0)strncpy(&wh_coded[i/3][0],"1101001",7);
 }
 char ilv[119]; 
 int k=0;
 for(int i=0; i<7; i++)
 {
  for(int j=0; j<17; j++) {ilv[k]=wh_coded[j][i];k++;}
 }
// tx="11"; 
 tx[0]='1'; tx[1]='1';tx_pos=2;
 for(int i=0; i<119; i++)
 {
  if(ilv[i]=='0'){tx[tx_pos]='1';tx[tx_pos+1]='0';tx_pos=tx_pos+2;}// tx+="10"; 
  else {tx[tx_pos]='0';tx[tx_pos+1]='1';tx_pos=tx_pos+2;}//tx+="01";
 }
  tx[239]=0;
 return 239;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int wspr_coding(void)
{memset(tx,0,sizeof(tx)); tx_pos=0;
 char wsprcall[7];
 unsigned long poly1=0xF2D05351UL;
 unsigned long poly2=0xE4613C47UL;
 byte sync[]={1,1,0,0,0,0,0,0,1,0,0,0,1,1,1,0,0,0,1,0,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1,0,0,1,0,1,0,0,
0,0,0,0,1,0,1,1,0,0,1,1,0,1,0,0,0,1,1,0,1,0,0,0,0,1,1,0,1,0,1,0,1,0,1,0,0,1,0,0,1,0,
1,1,0,0,0,1,1,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,0,1,0,0,1,1,1,0,1,1,0,0,1,1,0,1,0,0,0,1,
1,1,0,0,0,0,0,1,0,1,0,0,1,1,0,0,0,0,0,0,0,1,1,0,1,0,1,1,0,0,0,1,1,0,0,0};

memset(wsprcall,0,sizeof(wsprcall));
strcpy(wsprcall,call);
  // подготовить позывной
 if(strlen(wsprcall)<6){
  if(!isdigit(wsprcall[2])) {memmove(&wsprcall[1],&wsprcall[0],5*sizeof(char));wsprcall[0]=' ';wsprcall[6]=0;}
 }
 byte len=strlen(wsprcall);
 switch(len)
 { case 3: wsprcall[3]=' ';wsprcall[4]=' ';wsprcall[5]=' ';wsprcall[6]=0;break;
   case 4: wsprcall[4]=' ';wsprcall[5]=' ';wsprcall[6]=0; break;
   case 5: wsprcall[5]=' ';wsprcall[6]=0; break;
 }
 if(!isdigit(wsprcall[2])){printError("BAD CALL"); return 0;}

 // сформировать сжатый callsign
 //char abc[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ ";
 char c[7];
 memset(c,0,sizeof(c));
 for(int k=0; k<6; k++){for(int i=0; i<37; i++) if(wsprcall[k]==abc[i]){c[k]=i;}}
 uint64_t N=0ULL;//(((((c[0])*36 + c[1]) * 10 + c[2]) * 27 + c[3]-10) * 27 + c[4]-10) * 27 + c[5]-10;
  N=c[1];
  N=N+c[0]*36;
  N=N*10;
  N=N+c[2];
  N=N*27;
  N=N+c[3]-10;
  N=N*27;
  N=N+c[4]-10;
  N=N*27;
  N=N+c[5]-10;

 if(!isalpha(qth[0]) || !isalpha(qth[1]) || !isdigit(qth[2]) || !isdigit(qth[3])){printError("BAD LOC"); return 0;}

 int nlong= 180 - 20*(qth[0]-64-1);
 int n20d= 2*(qth[2]-48+1);
 nlong=nlong-n20d;
 int lat= 10*(qth[1]-64-1) + (qth[3]-48);
 uint64_t M=((nlong+180)/2)*180 +lat;

 // Power Level
 
 M=M*128 + pwr +64;
 N= ((N<<22) | (M&0x3fffffULL)) & 0x3ffffffffffffULL;

 // Conv coding
 unsigned long reg=0UL;
 uint64_t mask=1ULL;
 mask=mask<<49;
 for (int i=0; i<50; i++)
 {  reg=reg<<1;
    int b=0;
    if(N&mask) b=1;
    reg=reg | b;
    unsigned long r=0UL;
    b=0;
    r=reg&poly1;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b){tx[tx_pos]='1';tx_pos++;}//tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;}//tx+="0";
    b=0;
    r=reg&poly2;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b){tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;} //tx+="0";
    mask=mask>>1;
   // N=N<<1;
 }
  for (int i=0; i<31; i++)
 {  reg=reg<<1;
    unsigned long r=0;
    int b=0;
    r=reg&poly1;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b) {tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else {tx[tx_pos]='0';tx_pos++;} //tx+="0";
    b=0;
    r=reg&poly2;
    for(int j=0; j<32; j++) b=b^((r>>j)&1);
    if(b) {tx[tx_pos]='1';tx_pos++;} //tx+="1"; 
    else  {tx[tx_pos]='0';tx_pos++;} //tx+="0";
 }

 //Interliving
 char d[163];
 int p=0;
 for(int i=0; i<256; i++)
 { int j=0;
   // bit reversing
   j= ((i&128)>>7) | ((i&64)>>5) | ((i&32)>>3) | ((i&16)>>1) | ((i&8)<<1) | ((i&4)<<3) | ((i&2)<<5) | ((i&1)<<7);
   if(j<162){d[j]=tx[p]; p++;}
   if(p==162) break;
 }

 //merge with sync vector
 memset(tx,0,sizeof(tx));tx_pos=0;
 for(int i=0; i<162; i++) {tx[tx_pos]=1+ 48+ sync[i]+2*(d[i]-48); tx_pos++;} 
 return 162;
}

//---------------------------------------------------------------------------
int qrss_coding(char s[])
{
 int len=0;
 char cc[7];
 tx_len=0;  memset(tx,0,sizeof(tx));
 len=strlen(s);
 char d[7];
 for(int i=0; i<len; i++)
 {
  strcpy(cc,char_to_dfcw(s[i]));
  int cc_len=strlen(cc);
  if(s[i]==' ') {strcpy(d,"0000"); //пауза между словами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
  }
  else{
    for(int k=0;k<cc_len;k++) { // символ
     if(cc[k]=='-') strcpy(d,"1110");
     if(cc[k]=='_') strcpy(d,"10");
     if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
     strcat(tx,d);
    }// символ кончился
   strcpy(d,"00");//пауза между символами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
   }
 }
 //сообщение кончилось - добавить паузу после него
 //  strcpy(d,"0000"); //пауза между словами
 //  if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
 //  strcat(tx,d);
   tx_len=strlen(tx);
   return tx_len;
}

int dfcw_coding(char s[])
{
 int len=0;
 char cc[7];
 tx_len=0; memset(tx,0,sizeof(tx));
 len=strlen(s);
 char d[7];
 for(int i=0; i<len; i++)
 {
  strcpy(cc,char_to_dfcw(s[i]));
  int cc_len=strlen(cc);
  if(s[i]==' ') {strcpy(d,"00000"); //пауза между словами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
  }
  else{
    for(int k=0;k<cc_len;k++) { // символ
     if(cc[k]=='-') strcpy(d,"2220");
     if(cc[k]=='_') strcpy(d,"1110");
     if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
     strcat(tx,d);
    }// символ кончился
   strcpy(d,"00");//пауза между символами
   if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
   strcat(tx,d);
   }
 }
 //сообщение кончилось - добавить паузу после него
 //  strcpy(d,"00000"); //пауза между словами
 //  if(strlen(tx)+strlen(d)>=TX_LEN_MAX-1){tx_len=strlen(tx);return tx_len;}
 //  strcat(tx,d);
   tx_len=strlen(tx);
   return tx_len;
}

bool enable32(void)
{
  if(GMDDS)return true;
  bool result=false;
  Serial.write("4",1);
  result=readF();
  return result;
}

bool enable24(void)
{ if(GMDDS)return false;
  bool result=false;
  Serial.write("3",1);
  result=readF();
  return result;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void timer1(){
  if(tx_pos==0){mode=MODE_TX; paintTX(); pttON(); delay(1000); toneON();} //2c - пауза перед WSPR. Не помешает и другим модам.
  if(mode!=MODE_TX)return;
  if((tx_pos>0 && tx[tx_pos]!=tx[tx_pos-1]) || tx_pos==0){
  switch(mode_old)
  {case MODE_QRSS: case MODE_OP32:
    if(tx[tx_pos]=='0') toneOFF();else toneON();
    break;
   case MODE_DFCW: 
    if(tx[tx_pos]=='0') toneOFF();
    else{
    Code=CodeDFCW+(tx[tx_pos]-0x30-1)*tx_dc;
    setF(Code/*,false*/);
    toneON();
    }
    break;
   case MODE_WSPR:
    Code=CodeWSPR+(tx[tx_pos]-0x30-1)*tx_dc;
    if(DDS_VER>0 && !BIT32) setFastF(tx[tx_pos]-0x30-1);
    else setF(Code/*,false*/);
    break;
   case MODE_WSPR15:
    Code=CodeWSPR15+(tx[tx_pos]-0x30-1)*tx_dc;
    if(DDS_VER>0 && !BIT32) setFastF(tx[tx_pos]-0x30-1);
    else setF(Code/*,false*/);
    break;
   default: ;
  }
  }
  tx_pos++;
  if(tx_pos>=tx_len){
    tx_len=0; toneOFF();pttOFF();
    if(BEACON) {bcn_pos++; start_beacon(bcn_pos);}
    else {mode=mode_old; mode_old=MODE_TX;}
  }
}

#define LF2_FLAG_PIN A3 // умножать частоту на 2 (для передатчиков с делителем)
void setup() {
  // put your setup code here, to run once:
  setup_AD98xx_pins();
  freq=137500.0;
  set24bit();
  Serial.begin(9600);
  Serial.setTimeout(100);
  lcd.begin(16, 2);
  lcd.print(F("RN3AUS DDS CTRL"));
  lcd.setCursor(2,1);
  lcd.print(F("VERSION 1.0"));
  //set24bit();
  //set32bit();
  Code=1982464UL;
  //Code=553648128UL;
  
  EEPROM.get(EEPROM_CALL_ADR,call);
  if(call[0]<0 || call[0]>125) memset(call,0,sizeof(call));
  call_pos=strlen(call);
  EEPROM.get(EEPROM_QTH_ADR,qth);
  if(qth[0]<0 || qth[0]>125) memset(qth,0,sizeof(qth));
  qth_pos=strlen(qth);
  EEPROM.get(EEPROM_PWR_ADR,pwr);
  if(pwr>60 || pwr<0) pwr=0;
  for(int i=0; i<19; i++) if(pwr==pwr_list[i]) n_pwr=i;
  
  EEPROM.get(EEPROM_CODE_ADR,freq); Code=(unsigned long)(freq/Step);
  
  EEPROM.get(EEPROM_MSG_QRSS_ADR,msgq);
  if(msgq[0]<0 || msgq[0]>125) memset(msgq,0,sizeof(msgq));
  msgq_pos=strlen(msgq);
  
  EEPROM.get(EEPROM_MSG_DFCW_ADR,msgd);
  if(msgd[0]<0 || msgd[0]>125) memset(msgd,0,sizeof(msgd));
  msgd_pos=strlen(msgd);
  
 // EEPROM.get(EEPROM_MODE_ADR,mode);
 // if(mode<1)mode=1; if(mode>OPERATION_MODES)mode=OPERATION_MODES;

  EEPROM.get(EEPROM_CRYSTAL_ADR,Crystal);
  if(Crystal<0UL || Crystal>20000000UL)Crystal=12800000UL;
  for (byte i=0; i<CRYSTAL_LIST_N; i++) {if(Crystal==Crystal_list[i]) list_n=i;}

  EEPROM.get(EEPROM_CODEQRSS_ADR,freqQRSS); CodeQRSS=(unsigned long)(freqQRSS/Step);
  EEPROM.get(EEPROM_CODEDFCW_ADR,freqDFCW); CodeDFCW=(unsigned long)(freqDFCW/Step);
  EEPROM.get(EEPROM_CODEWSPR_ADR,freqWSPR); CodeWSPR=(unsigned long)(freqWSPR/Step);
  EEPROM.get(EEPROM_CODEOP32_ADR,freqOP32); CodeOP32=(unsigned long)(freqOP32/Step);
  EEPROM.get(EEPROM_CODEWSPR15_ADR,freqWSPR15);CodeWSPR15=(unsigned long)(freqWSPR15/Step);
  
  EEPROM.get(EEPROM_DOTQRSS_ADR,dot_qrss);
  if(dot_qrss>6000 || dot_qrss<=0)dot_qrss=10;
  
  EEPROM.get(EEPROM_DOTDFCW_ADR,dot_dfcw);
  if(dot_dfcw>6000 || dot_dfcw<=0)dot_dfcw=90;
  
  EEPROM.get(EEPROM_BCNLIST_ADR,bcn_list);
  if(bcn_list[0]==(char)0xff) memset(bcn_list,0,sizeof(bcn_list));

  EEPROM.get(EEPROM_LF2_ADR,LF2);
  if(LF2==0xff)LF2=0;
  //if(analogRead(LF2_FLAG_PIN)<10) LF2=true;
  
  delay(1000);
  clearLine(0);
  lcd.print(F("SEARCHING DDS..."));
  clearLine(1);
  
  bool result=false;
  
  if(analogRead(AD98XX_FLAG_PIN)<10) 
  {DDS_VER=0; DDSAD=true;
   set28bit();
   lcd.setCursor(0,0);lcd.print(F("DDS AD98xx FOUND"));
   goto M4;
  }
  
  for (int i=0; i<3; i++)
  { if(i==0)Serial.write("H");else Serial.write("?");
    result=readF();
    if(result) break;
    delay(300);
  }
  if(result){
     clearLine(0);
     lcd.setCursor(3,0);lcd.print(F("DDS VER "));
     lcd.print(DDS_VER);
     delay(1000);
     if(DDS_VER>=3) {set32bit(); enable32();}
  }
  else
  {
    setgmdds();
    for(int i=0; i<3; i++)
    { 
     result=setF(Code/*,false*/);
     if(result) break;
     delay(300);
    }
    clearLine(0);
    if(result){lcd.setCursor(2,0);lcd.print(F("GM DDS FOUND")); delay(1000);}
  }
  if(!result) {clearLine(0);lcd.print(F("DDS NOT DETECTED")); delay(2000);
               Serial.begin(9600); Serial.setTimeout(100);set24bit();
  }

M4:  delay(1000);
  if(LF2){lcd.setCursor(4,1);lcd.print(F("Fout x 2")); delay(1000);}
  clearLine(0);clearLine(1);
  toneOFF();pttOFF();setF(Code/*,false*/);
  //mode_old=mode;
  mode=MODE_TIME;
  printMode(mode);
  //freq=Step*Code;
}


unsigned long t_last_updw=0;
unsigned long t_last_tx=0;
unsigned int tx_pos_old=0;


bool waitToStart(char start_mode)
{ bool result=false;
  if(t_sec!=1) return false;
  if(start_mode==MODE_WSPR) {
   if(t_min%2==0) result=true;
  }
  if(start_mode==MODE_WSPR15){
   if(t_min%15==0) result=true;
  }
  if(!result) return false;
  mode=start_mode; printMode(mode);
  mode=MODE_TX; mode_old=start_mode; tx_pos=0; t_last_tx=millis();timer1();paintTX();
  return true;
}


void paintRotatingDash(void)
{
 static char d[]=": ";
 static byte cnt;
 static uint32_t last_dash;
 if(millis()-last_dash<500)return;
 last_dash=millis();
 lcd.setCursor(10,0);
 lcd.print(d[cnt]);     
 cnt++;cnt=cnt%2;;
}

void printBcnMenu(byte b)
{ lcd.setCursor(7,0);
  switch(b)
        { case BCN_LOAD: lcd.print(F("load")); break;
          case BCN_RUN : lcd.print(F("run ")); break;
          case BCN_QRSS: lcd.print(F("QRSS")); break;
          case BCN_DFCW: lcd.print(F("DFCW")); break;
          case BCN_OP32: lcd.print(F("OP32")); break;
          case BCN_WSPR: lcd.print(F("WSPR")); break;
          case BCN_WSPR15: lcd.print(F("WR15")); break;
          case BCN_RPT: lcd.print(F("RPT ")); break;
          case BCN_SAVE: lcd.print(F("save")); break;
          case BCN_CLR: lcd.print(F("clr ")); break;
          default: ;
        }
}
//////////////////////////////////////////////////////////////////////
void start_beacon(byte pos)
{
M3:
 if(pos>=strlen(bcn_list)){ //end of schedule
  mode=mode_old; toneOFF(); pttOFF(); tx_pos=0; tx_len=0;
  BEACON=false;
  return;
 }
 switch(bcn_list[pos])
 { case 'Q': printMode(MODE_QRSS);start_qrss(); break;
   case 'D': printMode(MODE_DFCW);start_dfcw(); break;
   case 'O': printMode(MODE_OP32);start_op32(); break; 
   case 'w': printMode(MODE_WSPR);start_wspr(); break;
   case 'W': printMode(MODE_WSPR15);start_wspr15(); break;
   case 'R': bcn_pos=0; pos=0; goto M3; break;
   default: return;
 }
 
 return;
}
//////////////////////////////////////////////////////////////////////
void start_qrss(void)
{      
       if(DDS_VER>=3){set32bit(); enable32();}
       freq=Step*CodeQRSS; freqQRSS=freq;
       //Code=CodeQRSS;
       setF(CodeQRSS/*, false*/);
       EEPROM.put(EEPROM_CODEQRSS_ADR,freqQRSS);
       tx_len=qrss_coding(msgq);
       tx_pos=0; tx_dc=0; mode_old=MODE_QRSS;mode=MODE_TX;
       mode_start=MODE_QRSS;
       timer_interval=1000UL * dot_qrss;
       if(timer_interval<=0)timer_interval=150UL;
       t_last_tx=millis();timer1();
       //paintTX();
}
void start_dfcw(void)
{      
       if(DDS_VER>=3){set32bit(); enable32();}
       freq=Step*CodeDFCW; freqDFCW=freq;
       //Code=CodeDFCW;
       setF(CodeDFCW/*, false*/);
       EEPROM.put(EEPROM_CODEDFCW_ADR,freqDFCW);
       tx_len=dfcw_coding(msgd);
       tx_pos=0;
       timer_interval=dot_dfcw*333UL;
       tx_df=5000.0/timer_interval;//4000.0 - сделал чуть шире стандарта для читаемости
       if(dot_dfcw>=60)tx_df=tx_df*dot_dfcw/60.0;
       tx_dc=tx_df/Step;
       if(tx_dc<1)tx_dc=1;       
       mode_old=MODE_DFCW;mode=MODE_TX;mode_start=MODE_DFCW;
       t_last_tx=millis();timer1();
       //paintTX();
}
void start_op32(void)
{      if(DDS_VER>=3){set32bit(); enable32();}
       freq=Step*CodeOP32; freqOP32=freq;
       //Code=CodeOP32;
       setF(CodeOP32/*, false*/);
       EEPROM.put(EEPROM_CODEOP32_ADR,freqOP32);
       tx_len=opera_coding();tx_pos=0;
       tx_dc=0;
       timer_interval=8192UL;  
       mode_old=MODE_OP32; mode=MODE_TX;mode_start=MODE_OP32;
       t_last_tx=millis();timer1();
      // paintTX();
}
void start_wspr(void)
{      if(DDS_VER>0 && DDS_VER<4){set24bit(); enable24();}
       if(DDS_VER>=4){set32bit(); enable32();}
       freq=Step*CodeWSPR; freqWSPR=freq;
       //Code=CodeWSPR;
       setF(CodeWSPR/*, false*/);
       EEPROM.put(EEPROM_CODEWSPR_ADR,freqWSPR);
       prepare_wspr(1);
       mode_start=MODE_WSPR;mode_next=MODE_TX; mode=MODE_WAIT;
}
void prepare_wspr(byte mul)
{
       tx_len=wspr_coding();
       tx_pos=0;timer_interval=683UL * mul;
       //tx_df=1000.0/timer_interval;
       tx_df=1.464844 / double(mul);
       double df1=tx_df/Step;
       tx_dc=tx_df/Step;
       if(df1-double(tx_dc)>=0.5) tx_dc=tx_dc+1;
       if(tx_dc<1)tx_dc=1;
        // для старой версии записать четыре частоты в DDS
       if(DDS_VER>0 && DDS_VER<4)
       {unsigned long cd4[4];
        cd4[0]=Code;
        cd4[1]=Code + (unsigned long)(tx_df/Step);    if(df1-double(cd4[1]-Code)>=0.5) cd4[1]++;   if(cd4[1]<=cd4[0])cd4[1]=cd4[0]+1UL;
        cd4[2]=Code + (unsigned long)(2.0*tx_df/Step);if(df1-2.0*double(cd4[2]-Code)>=0.5) cd4[2]++; if(cd4[2]<=cd4[1])cd4[2]=cd4[1]+1UL;
        cd4[3]=Code + (unsigned long)(3.0*tx_df/Step);if(df1-3.0*double(cd4[3]-Code)>=0.5) cd4[3]++; if(cd4[3]<=cd4[2])cd4[3]=cd4[1]+1UL;
        setF4(cd4);
       }
  
}
void start_wspr15(void)
{
      if(DDS_VER>0 && DDS_VER<4){set24bit(); enable24();}
      if(DDS_VER>=4){set32bit(); enable32();}
      freq=Step*CodeWSPR15; freqWSPR15=freq;
      Code=CodeWSPR15;
      setF(CodeWSPR15/*, false*/);
      EEPROM.put(EEPROM_CODEWSPR15_ADR,freqWSPR15);
      prepare_wspr(8);
      mode_start=MODE_WSPR15;mode_next=MODE_TX; mode=MODE_WAIT;
}

///////////////////////////////////////////////////////////////////////
#define DREBEZG 50 //ms
#define ANALOG_ZERO 100 //порог ниже которого аналоговый вход считается соединенным с землей
#define QSK_TIME 500 //время QSK - через сколько мс отключится PTT по окончании нажатия CW

void loop() {
  // put your main code here, to run repeatedly

 if(mode==MODE_TX && (millis()-t_last_tx)>=timer_interval-10UL){delayMicroseconds((millis()-t_last_tx-timer_interval)*1000UL);t_last_tx=millis();timer1();}
 //if(mode==MODE_TX && (millis()-t_start_tx)>=timer_interval*tx_pos-10UL){delayMicroseconds((millis()-t_start_tx-timer_interval*tx_pos)*1000UL);t_last_tx=millis();timer1();}
static unsigned long last_time_print;
///// GPS DEBUG
//if(millis()%3000UL==0) Serial.println("$GPRMC,123519,A,3840.038,N,00919.000,W,022.4,084.4,230394,003.1,W*6A");
//if(millis()%3000UL==0) Serial.println("$GPRMC,144326.00,A,3840.0017737,N,00919.3291611,W,0.080,323.3,210307,0.0,E,A*20");
//if(millis()%3000UL==0) Serial.println("$GNRMC,143909.00,A,5107.0020216,N,11402.3294835,W,0.036,348.3,210307,0.0,E,A*31");
////
 if(millis()-last_time_print>=500UL){
  last_time_print=millis();
  uint32_t sec = (millis() + t_dif) / 1000UL;
  //int timeHours = (sec / 3600ul) % 24;
  t_min = (byte)((sec % 3600UL) / 60UL);
  t_sec = (byte)((sec % 3600UL) % 60UL);

  if(mode==MODE_WAIT) {printTime(t_min,t_sec);
    if(mode_next==MODE_TX)  waitToStart(mode_start);
  }
  if(mode==MODE_TX && timer_interval>1000)paintRotatingDash();
  if(mode==MODE_TIME && timeIsSet)printTime(t_min,t_sec);
  if(mode==MODE_QTH){
  lcd.setCursor(1,1);
  if(gps_ready) {lcd.print(F("GPS = ")); lcd.print(qth);}
  else {if(gps_detected) lcd.print(F("(GPS) "));
        else  lcd.print(F("      "));
       }
  }
 }
 
 if(Code!=Code_old){freq=Step*Code; printFreq(Code);Code_old=Code;} 
 if(mode==MODE_TX && (tx_pos!=tx_pos_old)){paintTX(); tx_pos_old=tx_pos;}
 if(mode!=MODE_TX && mode!=mode_old){printMode(mode);mode_old=mode;}
 if(mode==MODE_CTRL)
 { static unsigned long last_tone;
   if(IsToneOn) last_tone=millis();
   if(analogRead(A1)<ANALOG_ZERO){
      //if(button==BTN_SELECT){}//в субменю CW
      if(IsToneOn==0){if(!IsPttOn)pttON(); toneON();  printTonePtt();}
   }
   else {
      if(IsToneOn==1){toneOFF(); printTonePtt();}
      if(IsPttOn && (millis()-last_tone>QSK_TIME)){pttOFF();printTonePtt();}
   };
 }
 byte button = detectButton();
 static unsigned long tb, tb1;
 if(millis()-tb1<DREBEZG) button=btn_old;
 else tb1=millis();
 if(button==btn_old && (mode<=MODE_WSPR15 || mode==MODE_TIME) && (button==BTN_UP || button==BTN_DOWN)) //автоповтор
 {
  if(millis()-t_last_updw>250UL){t_last_updw=millis();btn_old=BTN_NONE;}
 }
  
 if(button!=btn_old && (millis()-tb>DREBEZG))
 {tb=millis();
  if(mode==MODE_TX)
  { if(button==BTN_SELECT)
    {   toneOFF(); pttOFF();
        tx_pos=0;tx_len=0;
        BEACON=false; bcn_pos=0;
         mode=mode_old; mode_old=MODE_TX;
         printMode(mode);
    }
  }
  else if(mode==MODE_WAIT && mode_next==MODE_TX && button==BTN_SELECT)
    {mode=mode_start;//чтобы можно было отменить ожидание WSPR
     printMode(mode);
    }
  else 
  switch (button) {
    case BTN_UP:
      if(mode<=MODE_WSPR15)
      {if(last_btn==BTN_UP) {updwcnt++; if(updwcnt<10) dc=1UL;}
       else {updwcnt=0; dc=1UL;}
       if(updwcnt!=0 && updwcnt%10==0) dc=dc*2; if(BIT32){if(dc>10000) dc=10000;} else{if(dc>1000) dc=1000;}
       if(mode==MODE_CTRL) {Code=Code+dc;printFreq(Code);} 
       if(mode==MODE_QRSS) {CodeQRSS=CodeQRSS+dc; freq=CodeQRSS*Step;if(freq>136180.0 && freq<137700.0) CodeQRSS=(unsigned long)(137700.0/Step); if(freq>137783.0)CodeQRSS=(unsigned long)(136168.0/Step); printFreq(CodeQRSS);}
       if(mode==MODE_DFCW) {CodeDFCW=CodeDFCW+dc; freq=CodeDFCW*Step;if(freq>136180.0 && freq<137700.0) CodeDFCW=(unsigned long)(137700.0/Step); if(freq>137783.0) CodeDFCW=(unsigned long)(136168.0/Step); printFreq(CodeDFCW);}
       if(mode==MODE_WSPR) {CodeWSPR=CodeWSPR+dc; if(CodeWSPR*Step>137600.0) CodeWSPR=(unsigned long)(137400.0/Step); printFreq(CodeWSPR);}
       if(mode==MODE_WSPR15) {CodeWSPR15=CodeWSPR15+dc; if(CodeWSPR*Step>137625.0) CodeWSPR=(unsigned long)(137600.0/Step);printFreq(CodeWSPR15);}
       if(mode==MODE_OP32) {CodeOP32=CodeOP32+dc; if(CodeOP32*Step>137550.0) CodeOP32=(unsigned long)(137450.0/Step);printFreq(CodeOP32);}
      }
      if(mode==MODE_MSG_QRSS || mode==MODE_MSG_DFCW)
      {
        m[m_pos]='-';
        msg_updw(); 
      }
      if(mode==MODE_CALL)
      { cc[cc_pos]='-';
        call_updw();
      }
      if(mode==MODE_QTH)
      { qq[qq_pos]='-';
        lcd.setCursor(11,0);
        lcd.print(qq);
        qq_pos++; if(qq_pos>=QQ_LEN_MAX-1)qq_pos=QQ_LEN_MAX-1;
      }
      if(mode==MODE_PWR)
      { n_pwr++; if(n_pwr>18)n_pwr=18;
        power_updw();
      }
      if(mode==MODE_CLR_EEPROM)
      {
        clr_eeprom_updw();
      }
      if(mode==MODE_GEN)
      {
        list_n++;
        if(list_n>=CRYSTAL_LIST_N)list_n=0;//CRYSTAL_LIST_N-1;
        gen_updw(); 
      }
      if(mode==MODE_DOT_QRSS)
      {
        if(dot_qrss<10)dot_qrss++; 
        else {
           if(dot_qrss<120)dot_qrss=dot_qrss+10;
           else dot_qrss=dot_qrss+60;
        }
        if(dot_qrss>600)dot_qrss=600;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
      }
      if(mode==MODE_DOT_DFCW)
      {
        if(dot_dfcw<10)dot_dfcw++; 
        else {
          if(dot_dfcw<120)dot_dfcw=dot_dfcw+10;
          else dot_dfcw=dot_dfcw+60;
        }
        if(dot_dfcw>600)dot_dfcw=600;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_dfcw);lcd.print(F("s"));
      }
      if(mode==MODE_TIME && !timeIsSet)
      {  t_set_min++; t_set_min=t_set_min%60;
         t_set_sec=0;
         printTime(t_set_min,t_set_sec);
      }
      if(mode==MODE_BEACON)
      { 
        bcn++; bcn=bcn%BCN_MAX;
        printBcnMenu(bcn);
      }
      if(mode==MODE_LF2)
      {
        if(LF2)LF2=0;else LF2=1; lf2updw();
      }
      last_btn=button;
      break;
    case BTN_DOWN:
      if(mode<=MODE_WSPR15)
      {if(last_btn==BTN_DOWN) {updwcnt++;if(updwcnt<10) dc=1UL;}
       else {updwcnt=0; dc=1UL;}
       if(updwcnt!=0 && updwcnt%10==0) dc=dc*2; if(BIT32){if(dc>10000) dc=10000;} else{if(dc>1000) dc=1000;}
       if(mode==MODE_CTRL) {Code=Code-dc;printFreq(Code);} 
       if(mode==MODE_QRSS) {CodeQRSS=CodeQRSS-dc;freq=CodeQRSS*Step;if(freq>136180.0 && freq<137700.0) CodeQRSS=(unsigned long)(136180.0/Step); if(freq<136168.0)CodeQRSS=(unsigned long)(137783.0/Step);printFreq(CodeQRSS);}
       if(mode==MODE_DFCW) {CodeDFCW=CodeDFCW-dc;freq=CodeDFCW*Step;if(freq>136180.0 && freq<137700.0) CodeDFCW=(unsigned long)(136180.0/Step); if(freq<136168.0)CodeDFCW=(unsigned long)(137783.0/Step);printFreq(CodeDFCW);}
       if(mode==MODE_WSPR) {CodeWSPR=CodeWSPR-dc;if(CodeWSPR*Step<137400.0) CodeWSPR=(unsigned long)(137600.0/Step);printFreq(CodeWSPR);}
       if(mode==MODE_WSPR15) {CodeWSPR15=CodeWSPR15-dc;if(CodeWSPR15*Step<137600.0) CodeWSPR15=(unsigned long)(137620.0/Step);printFreq(CodeWSPR15);}
       if(mode==MODE_OP32) {CodeOP32=CodeOP32-dc;if(CodeOP32*Step<137450.0) CodeOP32=(unsigned long)(137550.0/Step);printFreq(CodeOP32);}
      }
      if(mode==MODE_MSG_QRSS || mode==MODE_MSG_DFCW)
      { m[m_pos]='_';
        msg_updw();
      }
      if(mode==MODE_CALL)
      { cc[cc_pos]='_';
        call_updw();
      }
      if(mode==MODE_QTH)
      { qq[qq_pos]='_';
        lcd.setCursor(11,0);
        lcd.print(qq);
        qq_pos++; if(qq_pos>=QQ_LEN_MAX-1)qq_pos=QQ_LEN_MAX-1;
      }
      if(mode==MODE_PWR)
      { n_pwr--; if(n_pwr<=0)n_pwr=0;
        power_updw();
      }
      if(mode==MODE_CLR_EEPROM){ clr_eeprom_updw();}
      if(mode==MODE_GEN)
      { if(list_n==0) list_n=CRYSTAL_LIST_N-1; else list_n--;
        gen_updw();
      }
      if(mode==MODE_DOT_QRSS)
      {
        if(dot_qrss<=10)dot_qrss--;
        else if(dot_qrss<=120)dot_qrss=dot_qrss-10;
             else dot_qrss=dot_qrss-60;
        if(dot_qrss<=0)dot_qrss=0;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_qrss);lcd.print(F("s"));
      }
      if(mode==MODE_DOT_DFCW)
      {
        if(dot_dfcw<=10)dot_dfcw--; 
        else if(dot_dfcw<=120)dot_dfcw=dot_dfcw-10;
             else dot_dfcw=dot_dfcw-60;
        if(dot_dfcw<=1)dot_dfcw=1;
        clearLine(1);lcd.setCursor(12,1); lcd.print(dot_dfcw);/*lcd.setCursor(15,1);*/lcd.print(F("s"));
      }
      if(mode==MODE_TIME && !timeIsSet)
      {  t_set_min--;if(t_set_min<0)t_set_min=59; t_set_min=t_set_min%60;
         t_set_sec=0;
         printTime(t_set_min,t_set_sec);
      }
      if(mode==MODE_BEACON)
      { 
        if(bcn==0) bcn=BCN_MAX-1; else bcn--; 
        bcn=bcn%BCN_MAX;
        printBcnMenu(bcn);
      }
      if(mode==MODE_LF2)
      {
        if(LF2)LF2=0;else LF2=1; lf2updw();
      }
      last_btn=button;
      break;
    case BTN_LEFT:
      if(mode==MODE_CTRL){toneOFF();pttOFF();}
      mode=mode-1;
      if(!submenu && mode<=0)mode=MODE_SUBMENU;
      if(submenu && mode<MODE_SUBMENU) mode=MODE_MAX-1;
      last_btn=button;
      break;
    case BTN_RIGHT:
      if(mode==MODE_CTRL){toneOFF();pttOFF();}
      mode=mode+1;
      if(submenu && mode>=MODE_MAX) mode=MODE_SUBMENU;
      if(!submenu && mode>MODE_SUBMENU)mode=1;
      last_btn=button;
      break;
    case BTN_SELECT:    
      if(mode==MODE_CTRL)
      {freq=Step*Code;
       setF(Code/*, false*/);
       EEPROM.put(EEPROM_CODE_ADR,freq);
      }
      if(mode==MODE_QRSS)
      {
       start_qrss();
      }
      if(mode==MODE_DFCW)
      {
       start_dfcw();
      }
      if(mode==MODE_WSPR)
      {
         start_wspr();
      }
      if(mode==MODE_WSPR15)
      {
       start_wspr15();
      }
      if(mode==MODE_OP32)
      {
       start_op32();
      }
      
      if(mode==MODE_MSG_QRSS)
      { clearLine(0); lcd.print(F("MSG QRSS:"));
        msg_select(msgq,msgq_pos);
        EEPROM.put(EEPROM_MSG_QRSS_ADR,msgq);
      }
      if(mode==MODE_MSG_DFCW)
      { clearLine(0); lcd.print(F("MSG DFCW:"));
        msg_select(msgd,msgd_pos);
        EEPROM.put(EEPROM_MSG_DFCW_ADR,msgd);
      }
      if(mode==MODE_CALL)
      {
        call[call_pos]=dfcw_to_char(cc);
        if(call[call_pos]==0 || cc_pos==0){if(cc_pos==0)call_pos--;if(call_pos<0)call_pos=0;call[call_pos]=0;}//ошибочный ввод или стереть
        else {call_pos++; if(call_pos>=CALL_LEN_MAX)call_pos=CALL_LEN_MAX-1;call[call_pos+1]=0;}
        clearLine(0); lcd.print(F("CALL:"));
        clearLine(1); lcd.print(call);
        cc_pos=0; memset(cc,0,sizeof(cc));
        EEPROM.put(EEPROM_CALL_ADR,call);
      } 
      if(mode==MODE_QTH)
      {
        qth[qth_pos]=dfcw_to_char(qq);
        if(qth_pos<2) if(!isalpha(qth[qth_pos]))qth[qth_pos]=0;
        if(qth_pos==2 || qth_pos==3) if(!isdigit(qth[qth_pos]))qth[qth_pos]=0;
        if(qth[qth_pos]==0 || qq_pos==0){if(qq_pos==0)qth_pos--;if(qth_pos<0)qth_pos=0;qth[qth_pos]='_';}//ошибочный ввод или стереть
        else {qth_pos++; if(qth_pos>=QTH_LEN_MAX)qth_pos=QTH_LEN_MAX-1;qth[qth_pos+1]=0;}
        clearLine(0); lcd.setCursor(7,0); lcd.print(F("LOC:"));
        clearLine(1); lcd.setCursor(7,1); lcd.print(qth);
        qq_pos=0; memset(qq,0,sizeof(qq));
        EEPROM.put(EEPROM_QTH_ADR,qth);
      }
      if(mode==MODE_CLR_EEPROM && clr_eeprom)
      { byte b=0xFF;
        for (int i = 0 ; i < EEPROM.length() ; i++) {EEPROM.write(i, b);}
        clr_eeprom=0;
        clearLine(1);lcd.print(F("EEPROM is empty"));
      }
      if(mode==MODE_GEN)
      {
        Crystal=Crystal_list[list_n];
        gen_updw();
        lcd.print(F(" OK"));
        EEPROM.put(EEPROM_CRYSTAL_ADR,Crystal);
        if(DDSAD) set28bit();
        else{
         if(BIT32)set32bit(); else set24bit(); 
        }
        freq=Step*Code;
      }
      if(mode==MODE_PWR)
      {
        EEPROM.put(EEPROM_PWR_ADR,pwr);
        lcd.setCursor(9,1); lcd.print(F("OK"));
      }
      if(mode==MODE_DOT_QRSS)
      {
        EEPROM.put(EEPROM_DOTQRSS_ADR,dot_qrss);
        lcd.setCursor(9,1); lcd.print(F("OK"));
      }
      if(mode==MODE_DOT_DFCW)
      {
       EEPROM.put(EEPROM_DOTDFCW_ADR,dot_dfcw);
       lcd.setCursor(9,1); lcd.print(F("OK"));
      }
      if(mode==MODE_TIME)
      { if(timeIsSet) {timeIsSet=false;lcd.setCursor(12,1); lcd.print("* ");t_set_min=t_min; t_set_sec=t_sec;}
        else {
         setmytime();
         lcd.setCursor(12,1); lcd.print(F("OK"));
         }
      }
      if(mode==MODE_BEACON)
      {byte len=strlen(bcn_list);
        if(len>=BCN_LIST_LEN && bcn>=BCN_QRSS && bcn<=BCN_WSPR15)bcn_list[BCN_LIST_LEN-1]=0; 
        switch(bcn)
        { case BCN_LOAD: EEPROM.get(EEPROM_BCNLIST_ADR,bcn_list);
                        if(bcn_list[0]==(char)0xff) memset(bcn_list,0,sizeof(bcn_list));  
                        break;
          case BCN_RUN:  BEACON=true; bcn_pos=0; start_beacon(0);  break;
          case BCN_QRSS:   strcat(bcn_list,"Q"); break;
          case BCN_DFCW:   strcat(bcn_list,"D");break;
          case BCN_OP32:   strcat(bcn_list,"O");break;
          case BCN_WSPR:   strcat(bcn_list,"w");break;
          case BCN_WSPR15: strcat(bcn_list,"W");break;
          case BCN_RPT:    strcat(bcn_list,"R");break;
          case BCN_SAVE: EEPROM.put(EEPROM_BCNLIST_ADR,bcn_list);lcd.setCursor(12,0); lcd.print(F("Ok  "));delay(500);lcd.setCursor(12,0); lcd.print(F("    "));break;
          case BCN_CLR: len=strlen(bcn_list);if(len>0)bcn_list[len-1]=0; break;
        }
        printBeaconList();
      }
      if(mode==MODE_SUBMENU)
      {
       submenu=!submenu; 
       if(submenu) mode++; //else mode--;
       printMode(mode); 
      }
      if(mode==MODE_LF2)
      {
       EEPROM.put(EEPROM_LF2_ADR,LF2); lcd.setCursor(0,1); lcd.print(F("Ok"));
      }
      last_btn=button;
      break;
      
    default:
    
      break;
   }
 }
  btn_old=button;
}
/////////////////////////////////////////////////////////////////////
///// одинаковые операции при нажатии кнопок UP DOWN (экономим место)
/////////////////////////////////////////////////////////////////////
void power_updw(void){
  pwr=pwr_list[n_pwr];
  if(pwr>60)pwr=60;
  clearLine(1);  lcd.setCursor(12,1); lcd.print(pwr);lcd.print(F("dB"));
}
void clr_eeprom_updw(void){
  if(clr_eeprom){clr_eeprom=0;clearLine(1);lcd.setCursor(7,1); lcd.print(F("NO"));}
  else{clr_eeprom=1;clearLine(1);lcd.setCursor(2,1); lcd.print(F("YES [SELECT]"));}
}
void msg_updw(void){
  lcd.setCursor(11,0);
  lcd.print(m);
  m_pos++;if(m_pos>=M_LEN_MAX-1)m_pos=M_LEN_MAX-1;
}
void call_updw(void){
  lcd.setCursor(5,0);
  lcd.print(cc);
  cc_pos++; if(cc_pos>=CC_LEN_MAX-1)cc_pos=CC_LEN_MAX-1;
}
void msg_select(char * msg, char &msg_pos)
{
  msg[msg_pos]=dfcw_to_char(m);
  if(msg[msg_pos]==0 || m_pos==0){if(m_pos==0)msg_pos--;if(msg_pos<0)msg_pos=0;msg[msg_pos]=0;}//ошибочный ввод или стереть
  else {msg_pos++; if(msg_pos>=MSG_LEN_MAX)msg_pos=MSG_LEN_MAX-1;msg[msg_pos+1]=0;}
  clearLine(1); lcd.print(msg);
  m_pos=0; memset(m,0,sizeof(m));
}
void gen_updw(void){
  clearLine(1);lcd.print(Crystal_list[list_n]);lcd.print(F(" Hz"));
}
void lf2updw(void){
  clearLine(0); if(LF2)lcd.print(F("Fout=2*F(272kHz)")); else lcd.print(F("Fout=F (136kHz)"));
}
void setmytime(void){
  int dm=t_set_min-t_min;
  int ds=t_set_sec-t_sec;
  if(ds<0) {ds=ds+60;dm=dm-1;}
  if(dm<0) dm=dm+60;
  t_dif+=(dm*60UL + ds)*1000UL;
  timeIsSet=true;
}
